@@ -167,9 +167,6 @@ println!("There are {} backbone atoms within 3.5Aͦ of the atom at index 42", to
     clippy::nonminimal_bool,
     clippy::print_stdout,
     clippy::use_debug,
-    clippy::shadow_unrelated,
-    clippy::shadow_same,
-    clippy::shadow_reuse,
     clippy::missing_docs_in_private_items,
     clippy::unwrap_used,
     clippy::map_unwrap_or,
