@@ -44,17 +44,19 @@ impl Atom {
     pub fn new(
         hetero: bool,
         serial_number: usize,
-        atom_name: &str,
+        atom_name: impl Into<String>,
         x: f64,
         y: f64,
         z: f64,
         occupancy: f64,
         b_factor: f64,
-        element: &str,
+        element: impl Into<String>,
         charge: isize,
     ) -> Option<Atom> {
-        if valid_identifier(atom_name)
-            && valid_identifier(element)
+        let atom_name = atom_name.into();
+        let element = element.into();
+        if valid_identifier(&atom_name)
+            && valid_identifier(&element)
             && x.is_finite()
             && y.is_finite()
             && z.is_finite()
@@ -203,8 +205,9 @@ impl Atom {
     /// For PDB files the name can at most contain 4 characters.
     /// If the name is invalid an error message is provided.
     /// The name can only contain valid characters, the ASCII graphic characters (`char.is_ascii_graphic() || char == ' '`).
-    pub fn set_name(&mut self, new_name: &str) -> Result<(), String> {
-        if !valid_identifier(new_name) {
+    pub fn set_name(&mut self, new_name: impl Into<String>) -> Result<(), String> {
+        let new_name = new_name.into();
+        if !valid_identifier(&new_name) {
             Err(format!(
                 "New name has invalid characters for atom {} name {}",
                 self.serial_number, new_name
@@ -281,8 +284,9 @@ impl Atom {
     /// For PDB files the element can at most contain 2 characters.
     /// If the element is invalid an error message is provided.
     /// The element can only contain valid characters, the ASCII graphic characters (`char.is_ascii_graphic() || char == ' '`).
-    pub fn set_element(&mut self, new_element: &str) -> Result<(), String> {
-        if !valid_identifier(new_element) {
+    pub fn set_element(&mut self, new_element: impl Into<String>) -> Result<(), String> {
+        let new_element = new_element.into();
+        if !valid_identifier(&new_element) {
             Err(format!(
                 "New element has invalid characters for atom {} name {}",
                 self.serial_number, new_element
@@ -293,13 +297,26 @@ impl Atom {
         }
     }
 
-    /// Get the atomic number of this Atom. If defined it uses `self.element()`, otherwise it uses `self.name()`.
+    /// Get the atomic number of this Atom. If defined it uses `self.element()`, otherwise it uses `self.name()`, if that still does not
+    /// find anything it uses the first character of `self.name()` if that is one of "CHONS". This could potentially lead to misclassifications
+    /// but in most cases should result in the correct element. Otherwise, the element is a required part in the PDB file so not specifying it
+    /// is fair to lead to unspecified behaviour.
     /// It fails when `self.element()` is not an element in the periodic table, or if `self.element()` is undefined and `self.name()` is not an element in the periodic table.
     pub fn atomic_number(&self) -> Option<usize> {
         if !self.element.is_empty() {
             reference_tables::get_atomic_number(self.element())
+        } else if !self.name().is_empty() {
+            let name = reference_tables::get_atomic_number(self.name());
+            #[allow(clippy::unwrap_used)] // The name is not empty so it cannot fail
+            if name.is_none() && "CHONS".contains(self.name().chars().next().unwrap()) {
+                reference_tables::get_atomic_number(
+                    &self.name().chars().next().unwrap().to_string(),
+                )
+            } else {
+                name
+            }
         } else {
-            reference_tables::get_atomic_number(self.name())
+            None
         }
     }
 
@@ -312,11 +329,9 @@ impl Atom {
     /// It also fails when the atomic radius is not defined for the given atomic number, so if the atomic
     /// number is higher than 96.
     pub fn atomic_radius(&self) -> Option<f64> {
-        if let Some(s) = self.atomic_number() {
-            reference_tables::get_atomic_radius(s)
-        } else {
-            None
-        }
+        self.atomic_number()
+            .map(reference_tables::get_atomic_radius)
+            .flatten()
     }
 
     /// Get the van der Waals radius for this Atom in Å. The radius is defined up to element 'Es' or atomic number 99.
@@ -326,23 +341,21 @@ impl Atom {
     /// It also fails when the atomic radius is not defined for the given atomic number, so if the atomic
     /// number is higher than 99.
     pub fn vanderwaals_radius(&self) -> Option<f64> {
-        if let Some(s) = self.atomic_number() {
-            reference_tables::get_vanderwaals_radius(s)
-        } else {
-            None
-        }
+        self.atomic_number()
+            .map(reference_tables::get_vanderwaals_radius)
+            .flatten()
     }
 
     /// Gets the covalent bond radii for this Atom.
     /// The result is the radius for a single, double and triple bond, where the last two are optional. If the radius for a double bond is unknown the radius for a triple bond is also unknown.
-    /// All values are given in picometers.
+    /// All values are given in Å.
     /// Sources:
     ///  * P. Pyykkö; M. Atsumi (2009). "Molecular Single-Bond Covalent Radii for Elements 1-118". Chemistry: A European Journal. 15 (1): 186–197. <https://doi.org/10.1002/chem.200800987>
     ///  * P. Pyykkö; M. Atsumi (2009). "Molecular Double-Bond Covalent Radii for Elements Li–E112". Chemistry: A European Journal. 15 (46): 12770–12779. <https://doi.org/10.1002/chem.200901472>
     ///  * P. Pyykkö; S. Riedel; M. Patzschke (2005). "Triple-Bond Covalent Radii". Chemistry: A European Journal. 11 (12): 3511–3520. <https://doi.org/10.1002/chem.200401299>
     ///
     /// It fails if the atomic number of this Atom is not defined (see `self.atomic_number()`).
-    pub fn covalent_bond_radii(&self) -> Option<(usize, Option<usize>, Option<usize>)> {
+    pub fn covalent_bond_radii(&self) -> Option<(f64, Option<f64>, Option<f64>)> {
         self.atomic_number()
             .map(reference_tables::get_covalent_bond_radii)
     }
@@ -454,6 +467,9 @@ impl Atom {
     /// Checks if this Atom overlaps with the given atom. It overlaps if the distance between the atoms is
     /// less then the sum of the radius from this atom and the other atom. The used radius is (`atom.atomic_radius()`).
     ///
+    /// Note: the atomic radius used in the unbound radius, this is in most cases bigger than the bound radius
+    /// and as such can result in false positives.
+    ///
     /// It fails if for any one of the two atoms the radius (`atom.atomic_radius()`) is not defined.
     pub fn overlaps(&self, other: &Atom) -> Option<bool> {
         self.atomic_radius()
@@ -471,6 +487,9 @@ impl Atom {
     /// atoms or any of their copies given a crystal of the size of the given unit cell stretching out to
     /// all sides.
     ///
+    /// Note: the atomic radius used in the unbound radius, this is in most cases bigger than the bound radius
+    /// and as such can result in false positives.
+    ///
     /// It fails if for any one of the two atoms the radius (`atom.atomic_radius()`) is not defined.
     pub fn overlaps_wrapping(&self, other: &Atom, cell: &UnitCell) -> Option<bool> {
         self.atomic_radius()
@@ -481,6 +500,43 @@ impl Atom {
             })
             .flatten()
     }
+
+    /// Checks if this Atom overlaps with the given atom. It overlaps if the distance between the atoms is
+    /// less then the sum of the radius from this atom and the other atom. The used radius is `atom.covalent_bond_radii().0`.
+    ///
+    /// Note: the atomic radius used in the bound radius to a single atom, this is similar to the bound radius for double or
+    /// triple bonds but could result in incorrect results.
+    ///
+    /// It fails if for any one of the two atoms the radius (`atom.covalent_bond_radii()`) is not defined.
+    pub fn overlaps_bound(&self, other: &Atom) -> Option<bool> {
+        self.covalent_bond_radii()
+            .map(|self_rad| {
+                other
+                    .covalent_bond_radii()
+                    .map(|other_rad| self.distance(other) <= self_rad.0 + other_rad.0)
+            })
+            .flatten()
+    }
+
+    /// Checks if this Atom overlaps with the given atom. It overlaps if the distance between the atoms is
+    /// less then the sum of the radius from this atom and the other atom. The used radius is `atom.covalent_bond_radii().0`.
+    /// Wrapping around the unit cell if needed. Meaning it will give the shortest distance between the two
+    /// atoms or any of their copies given a crystal of the size of the given unit cell stretching out to
+    /// all sides.
+    ///
+    /// Note: the atomic radius used in the bound radius to a single atom, this is similar to the bound radius for double or
+    /// triple bonds but could result in incorrect results.
+    ///
+    /// It fails if for any one of the two atoms the radius (`atom.covalent_bond_radii()`) is not defined.
+    pub fn overlaps_bound_wrapping(&self, other: &Atom, cell: &UnitCell) -> Option<bool> {
+        self.covalent_bond_radii()
+            .map(|self_rad| {
+                other.covalent_bond_radii().map(|other_rad| {
+                    self.distance_wrapping(other, cell) <= self_rad.0 + other_rad.0
+                })
+            })
+            .flatten()
+    }
 }
 
 impl fmt::Display for Atom {
@@ -673,10 +729,6 @@ mod tests {
         a.set_serial_number(42);
         assert_eq!(a.serial_number(), 42);
         assert_eq!(a.atomic_number(), Some(6));
-        assert!(a.set_name("HOH").is_ok());
-        assert!(a.atomic_radius().is_none());
-        assert!(a.vanderwaals_radius().is_none());
-        assert!(a.covalent_bond_radii().is_none());
         a.set_charge(-1);
         assert_eq!(a.charge(), -1);
         assert_eq!(a.pdb_charge(), "1-".to_string());
@@ -687,11 +739,14 @@ mod tests {
         let a = Atom::new(false, 0, "H", 1.0, 1.0, 1.0, 0.0, 0.0, "", 0).unwrap();
         assert_eq!(a.atomic_radius(), Some(1.54));
         assert_eq!(a.vanderwaals_radius(), Some(1.20));
-        assert_eq!(a.covalent_bond_radii(), Some((32, None, None)));
+        assert_eq!(a.covalent_bond_radii(), Some((0.32, None, None)));
         let a = Atom::new(false, 0, "Cl", 1.0, 1.0, 1.0, 0.0, 0.0, "", 0).unwrap();
         assert_eq!(a.atomic_radius(), Some(2.06));
         assert_eq!(a.vanderwaals_radius(), Some(1.82));
-        assert_eq!(a.covalent_bond_radii(), Some((99, Some(95), Some(93))));
+        assert_eq!(
+            a.covalent_bond_radii(),
+            Some((0.99, Some(0.95), Some(0.93)))
+        );
     }
 
     #[test]
