@@ -249,6 +249,36 @@ impl<'a> Chain {
         }
     }
 
+    /// Find all hierarchies matching the given information
+    pub fn find(
+        &'a self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformerResidue<'a>> + '_ {
+        self.residues()
+            .map(move |r| (r, search.clone().add_residue_info(r)))
+            .filter(|(_r, search)| !matches!(search, Search::Known(false)))
+            .map(move |(r, search)| r.find(search).map(move |h| h.extend(r)))
+            .flatten()
+    }
+
+    /// Find all hierarchies matching the given information
+    pub fn find_mut(
+        &'a mut self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformerResidueMut<'a>> + '_ {
+        self.residues_mut()
+            .map(move |r| {
+                let search = search.clone().add_residue_info(r);
+                (r, search)
+            })
+            .filter(|(_r, search)| !matches!(search, Search::Known(false)))
+            .map(move |(r, search)| {
+                let r_ptr: *mut Residue = r;
+                r.find_mut(search).map(move |h| h.extend(r_ptr))
+            })
+            .flatten()
+    }
+
     /// Get the list of Residues making up this Chain.
     /// Double ended so iterating from the end is just as fast as from the start.
     pub fn residues(&self) -> impl DoubleEndedIterator<Item = &Residue> + '_ {
