@@ -177,6 +177,18 @@ impl Conformer {
         }
     }
 
+    /// Find all atoms matching the given information
+    pub fn find(&self, search: Search) -> impl DoubleEndedIterator<Item = &Atom> + '_ {
+        self.atoms()
+            .filter(move |a| search.add_atom_info(a).complete().unwrap_or(true))
+    }
+
+    /// Find all atoms matching the given information
+    pub fn find_mut(&mut self, search: Search) -> impl DoubleEndedIterator<Item = &mut Atom> + '_ {
+        self.atoms_mut()
+            .filter(move |a| search.add_atom_info(a).complete().unwrap_or(true))
+    }
+
     /// Get the list of atoms making up this Conformer.
     /// Double ended so iterating from the end is just as fast as from the start.
     pub fn atoms(&self) -> impl DoubleEndedIterator<Item = &Atom> + '_ {
