@@ -10,6 +10,7 @@ mod model;
 mod mtrix;
 mod pdb;
 mod residue;
+mod search;
 mod symmetry;
 mod unit_cell;
 
@@ -24,5 +25,6 @@ pub use model::Model;
 pub use mtrix::MtriX;
 pub use pdb::PDB;
 pub use residue::Residue;
+pub use search::*;
 pub use symmetry::Symmetry;
 pub use unit_cell::UnitCell;
