@@ -256,6 +256,36 @@ impl<'a> Model {
         }
     }
 
+    /// Find all hierarchies matching the given information
+    pub fn find(
+        &'a self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformerResidueChain<'a>> + '_ {
+        self.chains()
+            .map(move |c| (c, search.clone().add_chain_info(c)))
+            .filter(|(_c, search)| !matches!(search, Search::Known(false)))
+            .map(move |(c, search)| c.find(search).map(move |h| h.extend(c)))
+            .flatten()
+    }
+
+    /// Find all hierarchies matching the given information
+    pub fn find_mut(
+        &'a mut self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformerResidueChainMut<'a>> + '_ {
+        self.chains_mut()
+            .map(move |c| {
+                let search = search.clone().add_chain_info(c);
+                (c, search)
+            })
+            .filter(|(_c, search)| !matches!(search, Search::Known(false)))
+            .map(move |(c, search)| {
+                let c_ptr: *mut Chain = c;
+                c.find_mut(search).map(move |h| h.extend(c_ptr))
+            })
+            .flatten()
+    }
+
     /// Get the list of Chains making up this Model.
     /// Double ended so iterating from the end is just as fast as from the start.
     pub fn chains(&self) -> impl DoubleEndedIterator<Item = &Chain> + '_ {
