@@ -229,6 +229,40 @@ impl<'a> Residue {
         }
     }
 
+    /// Find all hierarchies matching the given information
+    pub fn find(
+        &'a self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformer<'a>> + '_ {
+        self.conformers()
+            .map(move |c| (c, search.clone().add_conformer_info(c)))
+            .filter(|(_c, search)| !matches!(search, Search::Known(false)))
+            .map(move |(c, search)| {
+                c.find(search)
+                    .map(move |a| hierarchy::AtomConformer::new(a, c))
+            })
+            .flatten()
+    }
+
+    /// Find all hierarchies matching the given information
+    pub fn find_mut(
+        &'a mut self,
+        search: Search,
+    ) -> impl DoubleEndedIterator<Item = AtomConformerMut<'a>> + '_ {
+        self.conformers_mut()
+            .map(move |c| {
+                let search = search.clone().add_conformer_info(c);
+                (c, search)
+            })
+            .filter(|(_c, search)| !matches!(search, Search::Known(false)))
+            .map(move |(c, search)| {
+                let c_ptr: *mut Conformer = c;
+                c.find_mut(search)
+                    .map(move |a| hierarchy::AtomConformerMut::new(a, c_ptr))
+            })
+            .flatten()
+    }
+
     /// Get the list of conformers making up this Residue.
     /// Double ended so iterating from the end is just as fast as from the start.
     pub fn conformers(&self) -> impl DoubleEndedIterator<Item = &Conformer> + '_ {
